function [gridNodes,phi]=CalcSF(xp,F,InitParVec1,InitParVec2,cond)

% INPUTS: xp -> particle location
%         F -> particle deformation gradient
%         InitParVec1,InitParVec2 -> two vectors defining initial particle
%                                    domain

% OUTPUTS: gridNodes -> effective grid nodes associated with the particle
%          phi -> CPDI shape function vector

global x conn gridSpac nCelly

if strcmp(cond,'MPM')
    
    % Compute the element numbers associated the particle
    ie=(ceil(xp(1,1)/gridSpac))*nCelly-floor(xp(2,1)/gridSpac);
    
    % Determine effective grid nodes associated with the particle
    gridNodes=conn(:,ie);

    % Compute natural coordinates of the particle
    xpe=[2*(xp(1,1)-x(1,conn(1,ie)))/gridSpac-1
        2*(xp(2,1)-x(2,conn(1,ie)))/gridSpac-1];

    % FEM shape functions associated with the particle
    phi=1/4*[(1-xpe(1))*(1-xpe(2)) (1+xpe(1))*(1-xpe(2)) ...
        (1+xpe(1))*(1+xpe(2)) (1-xpe(1))*(1+xpe(2))];
    
elseif strcmp(cond,'uGIMP') || strcmp(cond,'cpGIMP')
    
    % Determine the partcile domain sizes. Particle domains are assumed to
    % be rectangles aligned with the background grid
    if strcmp(cond,'uGIMP')
        LxGimp=2*InitParVec1(1);
        LyGimp=2*InitParVec2(2);
    elseif strcmp(cond,'cpGIMP')
        LxGimp=2*InitParVec1(1)*F(1,1);
        LyGimp=2*InitParVec2(2)*F(2,2);
    end
    
    % Compute the element numbers associated the particle
    ie=(ceil(xp(1,1)/gridSpac))*nCelly-floor(xp(2,1)/gridSpac);

    % Compute natural coordinates of the particle
    xpe=[2*(xp(1,1)-x(1,conn(1,ie)))/gridSpac-1
        2*(xp(2,1)-x(2,conn(1,ie)))/gridSpac-1];
    
    % Determine the extension of the particle domain in natural coordinates
    nx1=fix(xpe(1)-LxGimp/gridSpac);
    nx2=fix(xpe(1)+LxGimp/gridSpac);
    ny1=fix(xpe(2)-LyGimp/gridSpac);
    ny2=fix(xpe(2)+LyGimp/gridSpac);
    
    % Initial value assignment for the effective grid nodes
    gridNodes=zeros((nx2-nx1+1)*(ny2-ny1+1),1);
    
    % Loop over divided parts of the particle domain for calculation of
    % effective shape functions associated with the particle
    count=0;
    for j1=nx1:nx2
        for j2=ny1:ny2
            count=count+1;
            % Determine the current cell number
            ee=ie-j2+j1*nCelly;
            % Determine the particle domain in natural coordinates
            x1=max(-1,-2*j1+xpe(1)-LxGimp/gridSpac);
            x2=min(1,-2*j1+xpe(1)+LxGimp/gridSpac);
            y1=max(-1,-2*j2+xpe(2)-LyGimp/gridSpac);
            y2=min(1,-2*j2+xpe(2)+LyGimp/gridSpac);
            % Determine effective grid nodes associated with the particle
            gridNodes((count-1)*4+1:count*4,1)=conn(:,ee);
            % Calculate the effective shape functions
            phi(1,(count-1)*4+1:count*4)=gridSpac^2/64/LxGimp/LyGimp* ...
                [(2*x2-2*x1-x2^2+x1^2)*(2*y2-2*y1-y2^2+y1^2) ...
                (2*x2-2*x1+x2^2-x1^2)*(2*y2-2*y1-y2^2+y1^2) ...
                (2*x2-2*x1+x2^2-x1^2)*(2*y2-2*y1+y2^2-y1^2) ...
                (2*x2-2*x1-x2^2+x1^2)*(2*y2-2*y1+y2^2-y1^2)];
        end
    end

elseif strcmp(cond,'CPDI')

    % Compute two vectors defining the current particle domain
    ParVec1=F*InitParVec1;
    ParVec2=F*InitParVec2;

    % Compute coordinates of 4 corners of the domain associated with the particle
    xEx=zeros(2,4);
    xEx(:,1)=[xp(1)-ParVec1(1)-ParVec2(1);xp(2)-ParVec1(2)-ParVec2(2)];
    xEx(:,2)=[xp(1)+ParVec1(1)-ParVec2(1);xp(2)+ParVec1(2)-ParVec2(2)];
    xEx(:,3)=[xp(1)+ParVec1(1)+ParVec2(1);xp(2)+ParVec1(2)+ParVec2(2)];
    xEx(:,4)=[xp(1)-ParVec1(1)+ParVec2(1);xp(2)-ParVec1(2)+ParVec2(2)];

    % Compute 4 element numbers associated the 4 corners
    ie1=(ceil(xEx(1,1)/gridSpac))*nCelly-floor(xEx(2,1)/gridSpac);
    ie2=(ceil(xEx(1,2)/gridSpac))*nCelly-floor(xEx(2,2)/gridSpac);
    ie3=(ceil(xEx(1,3)/gridSpac))*nCelly-floor(xEx(2,3)/gridSpac);
    ie4=(ceil(xEx(1,4)/gridSpac))*nCelly-floor(xEx(2,4)/gridSpac);

    % Determine effective grid nodes associated with the particle
    gridNodes=[conn(:,ie1);conn(:,ie2);conn(:,ie3);conn(:,ie4)];

    % Compute natural coordinates of the 4 corners
    xpe1=[2*(xEx(1,1)-x(1,conn(1,ie1)))/gridSpac-1;2*(xEx(2,1)-x(2,conn(1,ie1)))/gridSpac-1];
    xpe2=[2*(xEx(1,2)-x(1,conn(1,ie2)))/gridSpac-1;2*(xEx(2,2)-x(2,conn(1,ie2)))/gridSpac-1];
    xpe3=[2*(xEx(1,3)-x(1,conn(1,ie3)))/gridSpac-1;2*(xEx(2,3)-x(2,conn(1,ie3)))/gridSpac-1];
    xpe4=[2*(xEx(1,4)-x(1,conn(1,ie4)))/gridSpac-1;2*(xEx(2,4)-x(2,conn(1,ie4)))/gridSpac-1];

    % FEM shape functions associated with the 4 corners
    phis1=1/4*[(1-xpe1(1))*(1-xpe1(2)) (1+xpe1(1))*(1-xpe1(2)) ...
        (1+xpe1(1))*(1+xpe1(2)) (1-xpe1(1))*(1+xpe1(2))];
    phis2=1/4*[(1-xpe2(1))*(1-xpe2(2)) (1+xpe2(1))*(1-xpe2(2)) ...
        (1+xpe2(1))*(1+xpe2(2)) (1-xpe2(1))*(1+xpe2(2))];
    phis3=1/4*[(1-xpe3(1))*(1-xpe3(2)) (1+xpe3(1))*(1-xpe3(2)) ...
        (1+xpe3(1))*(1+xpe3(2)) (1-xpe3(1))*(1+xpe3(2))];
    phis4=1/4*[(1-xpe4(1))*(1-xpe4(2)) (1+xpe4(1))*(1-xpe4(2)) ...
        (1+xpe4(1))*(1+xpe4(2)) (1-xpe4(1))*(1+xpe4(2))];

    % Compute the CPDI shape function vector
    phi=[phis1 phis2 phis3 phis4]/4;

end