%
% This code demonstrates the main vaersions of the material point method (MPM)
%  for analyzing large deformation problems:
% 1. The original MPM,
% 2. The generalized interpolation material point (GIMP) method, and
% 3. The Convected particle domain interpolation (CPDI) method.
%
% This code is associated with example 4.2 of the following paper:
% A. Sadeghirad, R.M. Brannon, and J. Burghardt, "A convected particle domain 
% interpolation method to extend applicability of the material point method 
% for problems involving massive deformations", International Journal for 
% Numerical Methods in Engineering 86: 1435-1456, 2011.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%          Section 1: Initialization          %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear; % free system memory
global x conn gridSpac nCelly % declare 4 global variables for using in 
                              % functions "CalcSF.m" and "CalcGradSF.m"

interpolator='CPDI'; % choose between: 'MPM', 'uGIMP', 'cpGIMP', and 'CPDI'

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mechanical properties of the bar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Lambda=0; % Lame' constant(1)
mu=1e7/2; % Lame' constant(2)
rho=1000; % initial density of the medium

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% input data about time steps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dt=5e-5; % time step size
totTimeSteps=400; % total number of time steps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initial data associated with the background grid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gridSpac=0.1; % grid spacing
nCellx=12; % # of background grid cells in x dir
nCelly=12; % # of background grid cells in y dir
numnod=(nCellx+1)*(nCelly+1); % total number of the grid nodes
numelem=nCellx*nCelly; % total number of the background elements
x=zeros(2,numnod); % grid coordinates
for i=1:nCellx+1 % grid node coordinates
    for j=1:nCelly+1
        x(:,((nCelly+1)*(i-1)+j))=[gridSpac*(i-1);1.2-gridSpac*(j-1)];
    end
end
conn=zeros(4,numelem); % grid connectivities
for j=1:nCellx % background grids has 4 cell columns
    for i=1:nCelly % background grids has 22 cell rows
        elemn=(j-1)*nCelly+i; % cell number
        conn(4,elemn)=elemn+(j-1);
        conn(1,elemn)=conn(4,elemn)+1;
        conn(2,elemn)=conn(1,elemn)+nCelly+1;
        conn(3,elemn)=conn(2,elemn)-1;
    end
end

maxx1=gridSpac;
maxx2=gridSpac;

xp=zeros(2,10); % particle coordinates
numpar=0;
for i=1:numelem % loop over grid cells
    for j=[1/4 3/4 1/4 3/4
            1/4 1/4 3/4 3/4] % loop over particle per cell
        xptemp=x(:,conn(1,i))+j*gridSpac;
        if (xptemp(1)-maxx1)^2+(xptemp(2)-maxx2)^2>0.4^2-1e-6 && ...
                (xptemp(1)-maxx1)^2+(xptemp(2)-maxx2)^2<0.6^2+1e-6 && ...
                (xptemp(1)-maxx1)>0 && (xptemp(2)-maxx2)>0
            numpar=numpar+1;
            xp(:,numpar)=xptemp;
        end
    end
end
vol0=gridSpac*gridSpac/4*ones(1,numpar); % initial particle volumes
InitParVec1=[gridSpac/4*ones(1,numpar);zeros(1,numpar)];
InitParVec2=[zeros(1,numpar);gridSpac/4*ones(1,numpar)];

rho0=rho*ones(1,numpar); % initial particle densities
Vp=zeros(2,numpar); % Initial particles velocities
for i=1:numpar
    xptemp=[xp(1,i)-maxx1;xp(2,i)-maxx2];
    Vp(:,i)=body_f(xptemp(1),xptemp(2),0,2);
end
clear X Y alpha xptemp
sigma=zeros(3,numpar); % initial particle stresses
F=zeros(2,2,numpar); % initial particle deformation gradient
F(1,1,:)=ones(1,1,numpar); F(2,2,:)=ones(1,1,numpar);
SumErrorDisp=0; SumErrorDispEx=0;
SumErrorEner=0; SumErrorEnerEx=0;
dbcx=[]; dbcy=[];
for i=1:numnod
    if abs(x(1,i))<gridSpac+1e-5
        dbcx=union(dbcx,i);
    end
    if abs(x(2,i))<gridSpac+1e-5
        dbcy=union(dbcy,i);
    end
end
Xp=xp;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%           Section 2: Loop over time steps         %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for n=1:totTimeSteps %
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % initial value assignment at the first of the current time step
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    fint=zeros(2,numnod); % internal force vector
    fext=zeros(2,numnod); % external force vector
    q=zeros(2,numnod); % linear momentum vector
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % update particle volume and density for the current time step
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    vol=zeros(1,numpar);
    rhop=zeros(1,numpar);
    for i=1:numpar  % loop over the particles
        detF=det(F(:,:,i)); % determinant of the particle deformation gradient
        vol(i)=detF*vol0(i); % current particle volume
        rhop(i)=1/detF*rho0(i); % current particle density
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % map data from the particles to the grid nodes: 
    %     mass, momentum and external force
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    m=zeros(1,numnod);
    for i=1:numpar % loop over the particles
        % call the function "CalcSF" for calculating the CPDI shape functions
        [nodes,phi]=CalcSF(xp(:,i),F(:,:,i),InitParVec1(:,i),InitParVec2(:,i), ...
            interpolator);
        count=0;
        for i5=nodes'
            count=count+1;
            % Calculate the grid node masses
            m(1,i5)=m(1,i5)+phi(1,count)*rhop(i)*vol(i);
            % Calculate the grid node momentums
            q(:,i5)=q(:,i5)+phi(1,count)*Vp(:,i)*rhop(i)*vol(i);
            % Calculate the grid node external forces
            xptemp=[Xp(1,i)-maxx1;Xp(2,i)-maxx2];
            fext(:,i5)=fext(:,i5)+phi(1,count)* ...
                body_f(xptemp(1),xptemp(2),(n-1)*dt,1)*rhop(i)*vol(i);
        end
    end
    clear alpha
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % enforce Dirichlet boundary conditions: phase(1) 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i=dbcx, q(1,i)=0; end
    for i=dbcy, q(2,i)=0; end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % calculate the internal forces at the grid nodes 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i=1:numpar % loop over the particles
        % call the function "CalcGradSF" for calculating the CPDI shape
        % function gradients
        [nodes,gphi]=CalcGradSF(xp(:,i),F(:,:,i),InitParVec1(:,i), ...
            InitParVec2(:,i),vol(i),interpolator);
        count=0;
        for i5=nodes'
            count=count+1;
            fint(:,i5)=fint(:,i5)+ ...
                [gphi(1,count) 0;0 gphi(2,count);gphi(2,count) gphi(1,count)]' ...
                *sigma(:,i)*vol(i);
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % calculate the grid nodes momentum rates
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    q1=fext-fint;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % enforce Dirichlet boundary conditions: phase(2) 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i=dbcx, q1(1,i)=0; end
    for i=dbcy, q1(2,i)=0; end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % update the grid node momentums
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    q=q+q1*dt;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % update the particle velocities and locations
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i=1:numpar % loop over the particles
        % call the function "CalcSF" for calculating the CPDI shape functions
        [nodes,phi]=CalcSF(xp(:,i),F(:,:,i),InitParVec1(:,i), ...
            InitParVec2(:,i),interpolator);
        count=0;
        for i5=nodes'
            count=count+1;
            if abs(m(i5))>1e-12
                % update the particle velocity
                Vp(:,i)=Vp(:,i)+phi(1,count)*(q1(:,i5)/m(i5))*dt;
                % update the particle location
                xp(:,i)=xp(:,i)+(phi(1,count)*(q(:,i5)/m(i5)))*dt;
            end
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % update the particle deformation gradients and stresses
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Vgn=zeros(2,numnod); % initial assignment to the grid node velocities
    for i=1:numpar % loop over the particles
        % call the function "CalcGradSF" for calculating the CPDI shape
        % function gradients
        [nodes,gphi]=CalcGradSF(xp(:,i),F(:,:,i),InitParVec1(:,i), ...
            InitParVec2(:,i),vol(i),interpolator);
        % calculate the grid node velocities
        for i5=nodes'
            if abs(m(i5))>1e-12
                Vgn(:,i5)=q(:,i5)/m(i5);
            else
                Vgn(:,i5)=[0;0];
            end
        end
        % initial value for the increment of the deformation gradient
        incrF=[1 0;0 1]+[gphi(1,:)*Vgn(1,nodes)'*dt gphi(2,:)*Vgn(1,nodes)'*dt
            gphi(1,:)*Vgn(2,nodes)'*dt gphi(2,:)*Vgn(2,nodes)'*dt];
        % update the particle deformation gradient
        F(:,:,i)=incrF*F(:,:,i);
        % compute the determinant of the particle deformation gradient
        detF=det(F(:,:,i));
        % compute the stress tensor based on the Neo-Hookean material model
        Ssigma=Lambda*log(detF)/detF*[1 0;0 1]+mu/detF*(F(:,:,i)*F(:,:,i)'-[1 0;0 1]);
        % compute the stress vector
        sigma(:,i)=[Ssigma(1,1);Ssigma(2,2);Ssigma(1,2)];
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % plot snapshot
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if fix(n/50)==n/50
        disp([num2str(n) ' of ' num2str(totTimeSteps)]);
        subplot(2,4,fix(n/50)); Snapshot;
    end
    
end
